include("modules/stickers/id49d3044d0d564c3da01748cdc8cb6a1e/sentinel/sentinel.js");

if (typeof(libJS.Registry) == "undefined")
{
    libJS.Registry = new function()
    {
        var infoObject = system.createObject("INFO");

        this.toBool = function(key,defValue)
        {
            var value = this.toString(key, defValue);

            return (typeof(value) == "boolean") ? value : libJS.toBool(value);
        }

        this.toString = function(key,defValue)
        {
            var value = infoObject.Value(key);

            if ((value != "") && (value != null))
            {
                return value;
            }

            if (defValue != null)
            {
                return defValue;
            }
            else
            {
                throw new Error( "Invalid registry value : '" + key + "'");  
            } 
        }

        this.toNum = function(key,defValue)
        {
            var value = this.toString( key, "");

            if (value)
            {
                value = parseInt( value.toLowerCase(), 10);

                if (isNaN(value) == false)
                {
                    return value;
                }
            }

            if (defValue != null)
            {
                return defValue;
            }
            else
            {
                throw new Error( "Invalid registry value : '" + key + "'");  
            } 
        }
    }  
}

var fso = system.createObject("FILESYSTEM");

// listener used to regenerate bubble in the application
function ConfCarrouselUpdatorListener(moduleNode)
{
    this.onSuccess = function(event)
    {
        var idModule = moduleNode.getAttribute("id");

        fso.copy( event.image, system.env.hypermenu.directory + "/img/common/" + idModule + ".png");

        moduleNode.setAttribute("icon_carrousel", idModule + ".png");
    }

    this.onFailure = function(event)
    {
        moduleNode.removeAttribute("icon_carrousel");

        system.trace("No carrousel img associated with the following module : " + moduleNode.getAttribute("id"));
    }
}

// listener used to set the path to the bubble as a file in the application cache
function CarrouselImageListener(moduleNode)
{
    this.onSuccess = function(event)
    {
        moduleNode.setAttribute("icon_carrousel", event.image);
    }

    this.onFailure = function(event)
    {
        moduleNode.removeAttribute("icon_carrousel");

        system.trace("No carrousel img associated with the following module : " + moduleNode.getAttribute("id"));
    }
}

// listen the event from the feed data then retrieve the bubble from the server corresponding to the feed
function Feed2BubbleListener(urlFeed, moduleNode)
{
    this.onSuccess = function(event)
    {
        var channelLinkNode = event.responseXML.selectSingleNode("/rss/channel/link");

        if (channelLinkNode != null)
        {
            readerEngine.getCarrouselImageFromProxy( channelLinkNode.text, new CarrouselImageListener(moduleNode), {force:false});
        }
        else
        {
            readerEngine.getCarrouselImageFromProxy( urlFeed, new CarrouselImageListener(moduleNode), {force:false});
        }
    }

    this.onFailure = function(event)
    {
        readerEngine.getCarrouselImageFromProxy( urlFeed, new CarrouselImageListener(moduleNode), {force:false});
    }
}

function isBubbleImgCached(imgPath)
{
    if ((imgPath == null) || (imgPath == ""))
    {
        return false;
    }

    return imgPath.indexOf( system.createObject("INFO").Value("ProductAppDataDir") + "/cache") == 0;
}

function DisableSentinelUntilReaderWakeupListener(reader,sentinel,moduleNode)
{
    var self = this;

    var isActive = sentinel.isActive;
    var setState = sentinel.setState;

    // restore sentinel state
    var restoreSentinel = function()
    {
        if (typeof(sentinel.disableState) == "undefined")
        {
            return;
        }

        system.trace( "restore sentinel state after wake up of the following reader  : " + reader.getUrl());

        /************************** restore sentinel state *************************************************/ 
        sentinel.getState = sentinel.getActive = sentinel.isActive = isActive;

        sentinel.setActive = sentinel.setState = setState;

        try
        {
            // read the data store in the xml sentinel config
            sentinel.setState( libJS.Xml.nodeToBool( moduleNode.selectSingleNode("settings/sentinel/active")));
        }

        catch(err)
        {}

        delete sentinel.disableState;
        /*******************************************************************************************************/

        // stop listen reader event
        reader.remListener( self);
    }

    // disable sentinel
    var disableSentinel = function()
    {
        if (typeof(sentinel.disableState) != "undefined")
        {
            return; // already disable
        }

        system.trace( "disable the sentinel before wakeup of the following reader : " + reader.getUrl());

        /********************** disable the sentinel without modify xml sentinel config **********************/ 
        var onChange = sentinel.onChange;

        sentinel.onChange = function(e) 
        {
            if (e.type == "active")
            {
                sentinel.onChange = onChange; return; // restore only onChange handler ; don't modify active config value 
            }
            else
            {
                onChange.call( sentinel, e);
            }
        }

        if (typeof(sentinel.fireParamChangeEvent) == "function") // new sentinel event management
        {
            var fireParamChangeEvent = sentinel.fireParamChangeEvent;

            sentinel.fireParamChangeEvent = function(paramName)
            {
                if (paramName == "active")
                {
                    sentinel.fireParamChangeEvent = fireParamChangeEvent; return; // restore only fireParamChangeEvent handler ; don't modify active config value 
                }
                else
                {
                    fireParamChangeEvent.call( sentinel, paramName);
                }
            } 
        }

        setState.call( sentinel, false);
        /*****************************************************************************************************/

        // redefine the "active" getter to return value in the config file
        sentinel.getActive = sentinel.getState = sentinel.isActive = function()
        {
            try
            {
                return libJS.Xml.nodeToBool( moduleNode.selectSingleNode("settings/sentinel/active"));
            }
            catch(err)
            {
                return false;
            }
        }

        // redefine the "active" setter to disable this management when user change "active" property
        sentinel.setActive = sentinel.setState = function(bActive)
        {
            bActive = libJS.toBool( bActive);

            if (this.getState() == bActive)
            {
                return;
            }

            restoreSentinel();

            setState.call( sentinel, bActive);
        }

        sentinel.disableState = true;
    }

    this.onTimeout = function(e)
    {
        restoreSentinel();
    }

    this.onFailure = this.onException = this.onSimilar = function(e)
    {}

    this.onSuccess = function(e)
    {
        restoreSentinel();
    }

    disableSentinel();
}

// return the last launch date of the application ; must be a part of a lib
function getLastLaunchDate()
{
    var timestamp = parseInt( system.createObject("INFO").Value("launchdate"));

    if (isNaN(timestamp) == false)
    {
        var date = new Date();

        date.setTime( timestamp);

        return date;
    }
    else
    {
        return new Date(0); // never launch
    }
}

// true if the left date is in the same day of the right date
function isSameDay( left, right)
{
    return (left.getDate() == right.getDate()) && (left.getMonth() == right.getMonth()) && (left.getYear() == right.getYear()); 
}

// create a timer that will trigger at the end of the application startup
var startupTimer = new function()
{
    libJS.ListenerContainer.call( this);

    // id (optional) : used to distinguish timer
    function Timer(id)
    {
        libJS.ListenerContainer.call( this);

        // time : time to wait in millisecs
        this.set = function(time)
        {
            libJS.jsScheduler.setTimeout( "startupTimer.get().fireEvent(\"onTimeout\", {id:\"" + id + "\"});", time, true)
        }
    }

    var m_object = null; 

    this.get = function()
    {
        if (!m_object)
        {
            m_object = new Timer("startup");
        }

        return m_object;
    }

    this.set = function(time)
    {
        if (m_object) m_object.set( time);
    }
}


//Init rss reader
system.addEventListener("AppStart", function() {
    system.trace("Debut rssreaderInit");

    if(!Tabs.isInitSupported) {
        Tabs.forEachTab(function(tab) {
            var id=tab.documentElement.getAttribute("id");
            var mods = Tabs.getTabByTabId(id).selectNodes("/tab/column/mod[@type = 'id49d3044d0d564c3da01748cdc8cb6a1e']");
            for(var j=0; nb = mods.length > j; ++ j) {
                ID49D3044D0D564C3DA01748CDC8CB6A1E.initialize(mods[j]);
            }
        });
    }

    // start a time out to disable this functionnality
    startupTimer.set( libJS.Registry.toNum( "sentinelAtWakeupTimeout", 60000));

    system.trace("Fin rssreaderInit");

    system.createObject("INFO").Value("launchdate") = (new Date()).getTime(); 
});


function RSSReaderConnection(mod) {
    this.id=mod.getAttribute("id");
}
RSSReaderConnection.prototype.setRefresh=function(b) {
		var sentinel = readerEngine.sentinel(this.id);
		if (!sentinel)
		{
			return null;
		}
    
		return sentinel.feed().setActive(b);    
}

ID49D3044D0D564C3DA01748CDC8CB6A1E={
    initialize:function(mod) {
        if(mod.getAttribute("state")=="disabled") 
            return;

        this.connections[mod.getAttribute("id")]=new RSSReaderConnection(mod);
        
        var rssFeed = readerEngine.initRssSentinel(mod);

        if (rssFeed == null) {
            return;
        }

        var tabName=mod.ownerDocument.documentElement.getAttribute("text");
        
        rssFeed.sentinel.addUserParam("category", tabName);
        rssFeed.sentinel.addUserParam("feed_type", "rss");
        rssFeed.sentinel.addUserParam("item_class", libJS.Xml.xpathToString( mod, "settings/itemClass", ""));
        rssFeed.sentinel.addUserParam("categorySettings", libJS.Xml.xpathToString( mod, "settings/sentinel/categorySettings", ""));

        // check if the management to disable the sentinel at first reader response is enable
        if ((rssFeed.sentinel.isActive() == true) && (isSameDay( new Date, getLastLaunchDate()) == false) && (libJS.Registry.toBool( "sentinelAtWakeup", false) == false)) 
        {
            var listener = new DisableSentinelUntilReaderWakeupListener( rssFeed.reader, rssFeed.sentinel, mod);

            rssFeed.reader.addListener( listener); // disable it when a new feed has been retrieved

            startupTimer.get().addListener( listener); // disable if when the startup step is finished
        }

        var iconCarrousel = mod.getAttribute("icon_carrousel");

        if ((system.env.hypermenu) && (isBubbleImgCached(iconCarrousel) == true) && (fso.fileExist( iconCarrousel) == false))
        {
            rssFeed.reader.engineRequest( new Feed2BubbleListener( rssFeed.reader.getUrl(), mod), {force:false});
        }
    },
    deinitialize:function(n) {        
        var id=n.getAttribute("id");
        
        this.connections[mod.getAttribute("id")].deinitialize(mod);
        delete this.connections[id];

        var sentinel=readerEngine.sentinel(id);
        if(sentinel) {
            system.trace("remove eventual the sentinels on the module "+id);
            sentinel.dispose();
        }
    },
    addToDesktop:function(param) {
         var tab=Tabs.getTabByTabId("windows");    
         var xml=new ActiveXObject("MSXML2.DOMDocument.4.0");
         var mid=system.getUniqueId();
         xml.loadXML('<mod type="id49d3044d0d564c3da01748cdc8cb6a1e" static="0" id="" state="disabled"><removeBtn>1</removeBtn><closeRestSentinelBtn>0</closeRestSentinelBtn><closeRemoveSentinelBtn>0</closeRemoveSentinelBtn><url option="2"></url><icon></icon><topmost></topmost><width></width><height></height><group></group><w></w><h></h><settings option="1"><backgroundColor>#FFFFFF</backgroundColor><nbItems option="1">4</nbItems><openUrl option="1">0</openUrl><getFavIcon>1</getFavIcon><getTitle>1</getTitle><interface>1</interface><displayMode option="1">shortnews</displayMode><sentinel option="1"><keyword></keyword><keyword></keyword><keyword></keyword><active>0</active><name/></sentinel></settings><x></x><y></y></mod>')
             xml.selectSingleNode("mod/url").text=unescape(param.url);
         xml.selectSingleNode("mod/@id").text=mid;
         tab.selectSingleNode("tab/column").appendChild(xml.selectSingleNode("mod"));
         Tabs.saveTabByTabId("windows");
         stickers.activateStickerById(mid, function() {}, {x:0,y:0});
     },
     createXMLNode:function(properties) {
        var n=new ActiveXObject("MSXML2.DOMDocument.4.0");
        n.loadXML("<mod type='id49d3044d0d564c3da01748cdc8cb6a1e' static='0' icon_carrousel='empty_feed.png' cid='protocol' id='' desktop='false'><title option='1'></title><url option='2'><![CDATA[]]></url><link></link><icon><![CDATA[]]></icon><topmost></topmost><width></width><height></height><group></group><w></w><h></h><settings option='1'><backgroundColor>#FFFFFF</backgroundColor><nbItems option='1'>4</nbItems><openUrl option='1'>0</openUrl><getFavIcon>1</getFavIcon><getTitle>1</getTitle><interface>1</interface><displayMode option='1'>shortnews</displayMode><sentinel option='1'><keyword></keyword><keyword></keyword><keyword></keyword><active>0</active><name></name></sentinel></settings><pubUrl>http://s.livesticker.com/c/d/clients/bubbledock/pub_sentinelle/pub_sentinelle.htm</pubUrl></mod>");
        n.selectSingleNode("mod/url").text=unescape(properties.href || properties.url);
        return n.documentElement;
     },
    getConnection:function(id)
	  {
        return this.connections[id];
    },
    connections:[]     
}

system.declareJScriptCommand("addRssDesktopSticker", "ID49D3044D0D564C3DA01748CDC8CB6A1E.addToDesktop");
