function getModuleVersion () {
    var d=system.createXmlDocument();
    d.load(system.createObject("INFO").Value("ProductAppDataDir")+"\\modules\\stickers\\id49d3044d0d564c3da01748cdc8cb6a1e\\manifest.xml");
    if(d.selectSingleNode("module/pkg/version")) {
        return d.selectSingleNode("module/pkg/version").text;
    }
}
/// Sentinel listener used to update the livesticker configuration 
function UpdateOldConfigListener(sentinelNode)
{
  this.onParamChanged = function(e)
  {
    var sentinel = e.source;
    
    try
    {
      // if the parameter who change is not in the sentinel definition, don't update the sentinel config file
      if ((e.attribute != null) && (e.attribute.definition == false))
      {
        return;
      }

      var filterDef = e.source.filter().definition();
      var matchDef  = e.source.match().definition();
      
      var sentinelDef = {};
      
      function set(key,value,def)
      {
        if (value != undefined)
        {
          if (def != undefined)
          {
            if (value != def) sentinelDef[key] = value;
          }
          else
          {
            sentinelDef[key] = value;
          }
        }
      }
      
      set( "active"   , sentinel.filter().isActive()); 
      set( "name"     , (sentinel.filter().getXsltParams() || {}).name); 
      set( "key"      , sentinel.match().getKey()  , "title"); 
      set( "keyword"  , sentinel.filter().getKeyword()); 
      set( "type"     , sentinel.filter().getType() , "rss_and"); 
      set( "xslt"     , sentinel.filter().getResultToPublishedDocumentXslt() , system.createObject("INFO").Value("IDC_ALERT_SENTINEL_XSLT")); 
      set( "pubType"  , sentinel.filter().getPublisher() , "rss_single"); 

      var xmlData = libJS.Xml.objectToData( "sentinel", sentinelDef);
      
      if (xmlData == "")
      {
        return;
      }
      
      var newSentinelNode = libJS.Xml.dataToNode( xmlData);
      
      libJS.Xml.mergeNode( sentinelNode, newSentinelNode);
      
      // sentinel -> settings -> mod -> column -> tab
      var tabNode = sentinelNode.selectSingleNode("ancestor::tab");
      
      if (tabNode == null) 
      {
        return;
      }
      
      Tabs.saveTabByTabId( tabNode.getAttribute("id"));
    }
    
    catch(err)
    {
      system.error(err);

      system.error("Unable to save the parameter of the following sentinel : " + e.source.id());
    }
  }
}


ReaderEngine.prototype.initRssSentinel = function(moduleNode)
{
    function createFeed()
    {
        // init values
		var sticker = {}; var feed = {}; var match = {}; var filter = {}; var action = {};

        var matchType = "rss"; var filterType = "rss_and"; var actionType = "alert";

        try
        {
            sticker.type = moduleNode.getAttribute("type");
            
            sticker.id = moduleNode.getAttribute("id");
            
            sticker.url = libJS.Xml.nodeToString( moduleNode.selectSingleNode("url"), "");
            
            if (!sticker.url)
            {
                return null;
            }
			
			try
			{
				var oUrl = new libJS.Url( sticker.url);
				
				if (oUrl.getProtocol() == "reg")
				{
					system.trace("[SENTINEL] Registry feed rss url detected. ('" + sticker.url + "')");
					
					regUrl = libJS.Registry.toString( oUrl.getHost());
					
					sticker.url = regUrl;
					
					system.trace("[SENTINEL] New feed rss url : '" + sticker.url + "'");
				}
			}
			catch(err)
			{}
            
            feed.preprocessorState  = true;
            feed.postprocessorState = true;
            
            match.processNewDocumentState = true;
            
            match.key = libJS.Xml.nodeToString( moduleNode.selectSingleNode("settings/sentinel/key"), "title");
            
            filterType = libJS.Xml.nodeToString( moduleNode.selectSingleNode("settings/sentinel/type"), "rss_and");
            
            filter.title = libJS.Xml.nodeToString( moduleNode.selectSingleNode("title"), "");
            
            filter.keyword = libJS.Xml.nodesToArray( moduleNode.selectNodes("settings/sentinel/keyword"));
            
            filter.keywordNodes = ["title","description"];
            
            filter.active = libJS.Xml.nodeToBool( moduleNode.selectSingleNode("settings/sentinel/active"), false); // disable by default
            
            actionType = (libJS.Xml.nodeToString( moduleNode.selectSingleNode("settings/sentinel/pubType"), "rss_single") == "rss_single") ? "alert" : "alertEx";
            
            var sentinel = readerEngine
                                .sentinel( sticker.id, {url:sticker.url, type:"rss"}, {owner:sticker.type}, {directory:libJS.getStickersPath() + "/" + sticker.type})
                                .feed( feed)
                                .match( matchType, match).filter( filterType, filter);
            
            if (actionType == "alert")
            {
                var alert = {};
            
                alert.xslt = libJS.Xml.nodeToString( moduleNode.selectSingleNode("settings/sentinel/alert_xslt"),"modules/stickers/id49d3044d0d564c3da01748cdc8cb6a1e/sentinel/alert_report.xslt");
                alert.width = 472;
                alert.height = 200;
                
                sentinel.doAlert(alert);
            }
            else
            {
                sentinel.doAction("alertEx",{xslt:libJS.Xml.nodeToString( moduleNode.selectSingleNode("settings/sentinel/xslt"), "IDS_ALERT_SENTINEL_XSLT")});
            }
            
            // Register user params used to custom sentinel result view 
            sentinel.param( "bar_title", libJS.Lang.toString( "IDS_ALERT_SENTINEL", "Alerte"));
            sentinel.param( "mid_id"   , sticker.id);
            sentinel.param( "language" , libJS.Registry.toString('lang'));
            sentinel.param( "mod_settings_thumbnailLink" , libJS.Xml.nodeToString(moduleNode.selectSingleNode("settings/thumbnailLink"), '0'));
            sentinel.param( "mod_cid" , libJS.Xml.nodeToString(moduleNode.selectSingleNode("@cid"), ''));
            sentinel.param( "mod_type" , libJS.Xml.nodeToString(moduleNode.selectSingleNode("@type"), ''));
            sentinel.param( "mod_revision" , getModuleVersion());
            sentinel.param( "libdock_revision" , LIBBOX_VERSION);
            sentinel.param( "text_read_item" , libJS.Lang.toString( "IDS_ALERT_NEXT", "Lire la suite"));
            //sentinel.param( "alert_expert_mode_txt" , libJS.Lang.toString( "IDS_ALERT_EXPERT_MODE", "Mode avancé"));
            sentinel.param( "sentinel_msg_1" , libJS.Lang.toString( "IDS_ALERT_MSG1", "M'avertir pour tout nouvel article"));
            sentinel.param( "sentinel_msg_2" , libJS.Lang.toString( "IDS_ALERT_MSG2", "M'avertir pour tout nouvel article contenant un des mots clés"));
            
            if (sentinel.getListenerById("oldconfig") == null)
            {
                sentinel.addListener(new UpdateOldConfigListener( moduleNode.selectSingleNode("settings/sentinel")), "oldconfig");
            }
            
            return new function()
            {
                this.reader   = sentinel.reader();
                this.sentinel = sentinel.filter();
            };
        }
        catch(err)
        {
    		system.error("Unable to create the following reader object : id = " + sticker.id + ", url=" + sticker.url);
    
    		system.error("Last error : " + error.message);
    		
    		return null;
        }
    }

    // don't use mySentinel document to store this sentinel
    //m_mySentinelsDocument.setActive( false);

    var rssFeed = createFeed();

    // use my sentinel document with other sentinels
    //m_mySentinelsDocument.setActive( true);
    
    return rssFeed;
}
