<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:media="http://search.yahoo.com/mrss/" xmlns:mediadm="http://search.yahoo.com/mrss" xmlns:itunes="http://www.itunes.com/dtds/podcast-1.0.dtd" xmlns:nsb="urn:functions:xslt" xmlns:msxsl="urn:schemas-microsoft-com:xslt">

    <msxsl:script language="JavaScript" implements-prefix="nsb">
        <![CDATA[
            function normalizeJScriptString(nodeList) {
                // escape quote then trim
                return (nodeList.length != 0) ? nodeList.nextNode().text.replace(/'/g, "\\\'").replace(/(?:^\s+|\s+$)/g, "") : "";
            }
        ]]>
    </msxsl:script>

    <xsl:variable name="audioFileTypes">,au,mp3,aac,mid,wav,wma,aif,gsm,m4a,snd,midi,aiff,</xsl:variable>
    <xsl:variable name="videoFileTypes">,qt,rm,ra,rv,flv,swf,mov,mpg,mp4,3gp,m4v,ram,asf,avi,wmv,rpm,qti,qif,bmp,tif,psd,3g2,smi,mpeg,qtif,tiff,smil,</xsl:variable>

    <xsl:template name="detectEnclosureType">
        <xsl:param name="type"/>
        <xsl:param name="url"/>
        <xsl:param name="flashplayer"/>
        
        <xsl:choose>
            <xsl:when test='$flashplayer != ""'>
                <xsl:text>flash</xsl:text>
            </xsl:when>
            <xsl:when test='contains($type, "audio/")'>
                <xsl:text>audio</xsl:text>
            </xsl:when>
            <xsl:when test='contains($type, "video/")'>
                <xsl:text>video</xsl:text>
            </xsl:when>
            <xsl:when test='contains($type, "image/") or contains($type, "img/")'>
                <xsl:text>image</xsl:text>
            </xsl:when>
        </xsl:choose>
    </xsl:template>

    <xsl:template name="webLink">
        <xsl:param name="link"/>
        
        <xsl:if test="($link != '') and ($link != '#')">
            <div class='webLink'>
            <a href="#null" onclick="window.external.navigate('{$link}')"><img type='nsb' src="{$imgPath}/{$language}/bt_link_rss.gif" /></a>
            </div>
        </xsl:if>
    </xsl:template>
  
    <xsl:template name="itemTitle">
        <xsl:param name="link"/>
        <xsl:param name="title"/>

        <span class='title'>
            <xsl:choose>
                <xsl:when test="$mod_settings_openUrl = '1' and $isWindowsSticker = '0'">
                    <a href="#1" onclick="App.getModule('{$mod_id}').contentObj.displayInterface('{$link}')">
                    <xsl:call-template name="normalizeString">
                        <xsl:with-param name="data">
                            <xsl:value-of select='$title' disable-output-escaping='yes'/>
                        </xsl:with-param>
                    </xsl:call-template>
                    </a>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:choose>
                        <xsl:when test="($link != '') and ($link != '#')">
                            <xsl:choose>
                                <xsl:when test="contains($link, 'javascript:nsx')">
                                    <a href="#null" link='{$link}' onclick="window.external.navigate('{$link}')">
                                        <xsl:call-template name="normalizeString">
                                            <xsl:with-param name="data">
                                                <xsl:value-of select='$title' disable-output-escaping='yes'/>
                                            </xsl:with-param>
                                        </xsl:call-template>
                                    </a>
                                </xsl:when>
                                <xsl:otherwise>
                                    <a href="#null" link='{$link}' onclick="window.external.navigate('{$link}')">
                                        <xsl:call-template name="normalizeString">
                                            <xsl:with-param name="data">
                                                <xsl:value-of select='$title' disable-output-escaping='yes'/>
                                            </xsl:with-param>
                                        </xsl:call-template>
                                    </a>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:call-template name="normalizeString">
                                <xsl:with-param name="data">
                                    <xsl:value-of select='$title' disable-output-escaping='yes'/>
                                </xsl:with-param>
                            </xsl:call-template>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:otherwise>
            </xsl:choose>
        </span>
    </xsl:template>

    <xsl:template name="itemIcon">
        <xsl:param name="type"/>
        <xsl:param name="url"/>
        <xsl:param name="title"/>
        <xsl:param name="image"/>
        <xsl:param name="class"/>
        <img class="ii {$class}" thumbnail="{$image}">
            <xsl:attribute name='src'>
                <xsl:choose>
                    <xsl:when test='($type = "audio") and ($url != "")'>
                        <xsl:choose>
                            <xsl:when test='$image != ""'>
                                <xsl:value-of select='$image' />
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:value-of select='$imgPath' /><xsl:text>/bt_small_play.gif</xsl:text>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:when>
                    <xsl:when test='($type = "video") and ($url != "")'>
                        <xsl:choose>
                            <xsl:when test='$image != ""'>
                                <xsl:value-of select='$image' />
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:value-of select='$imgPath' /><xsl:text>/bt_small_tv.gif</xsl:text>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:when>
                    <xsl:when test='($type = "flash") and ($url != "")'>
                        <xsl:choose>
                            <xsl:when test='$image != ""'>
                                <xsl:value-of select='$image' />
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:value-of select='$imgPath' /><xsl:text>/bt_small_tv.gif</xsl:text>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:when>
                    <xsl:when test='($type = "player") and ($url != "")'>
                        <xsl:choose>
                            <xsl:when test='$image != ""'>
                                <xsl:value-of select='$image' />
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:value-of select='$imgPath' /><xsl:text>/bt_small_tv.gif</xsl:text>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:choose>
                            <xsl:when test='$image != ""'>
                                <xsl:value-of select='$image' />
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:value-of select='$imgPath' /><xsl:text>/li.gif</xsl:text>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:attribute>
        </img>
    </xsl:template>

    <xsl:template name="itemEnclosure">
        <xsl:param name="type"/>
        <xsl:param name="url"/>
        <xsl:param name="title"/>
        <xsl:param name="link"/>
        
        <xsl:if test='$url != ""'>
            <xsl:choose>
                <xsl:when test='$type = "audio"'>
                    <img class="enclosure audio" src="{$imgPath}/{$language}/bt_ecouter_on.png" type="nsb" url="{$url}" mimetype="{$type}" alt="{$title}" title="{$title}"/>
                    <!--<img class="enclosure download" src="{$imgPath}/{$language}/bt_telecharger_on.png" type="nsb" url="{$url}" mimetype="{$type}" alt="{$title}" title="{$title}"/>-->
                </xsl:when>
                <xsl:when test='$type = "video"'>
                    <img class="enclosure video" src="{$imgPath}/{$language}/bt_video_on.png" type="nsb" url="{$url}" mimetype="{$type}" alt="{$title}" title="{$title}"/>
                </xsl:when>
                <xsl:when test='$type = "flash"'>
                    <img class="enclosure flash" src="{$imgPath}/{$language}/bt_video_on.png" type="nsb" url="{$url}" mimetype="{$type}" alt="{$title}" title="{$title}"/>
                </xsl:when>
                <xsl:when test='$type = "image"'>
                    <xsl:call-template name="itemReadAll">
                        <xsl:with-param name="link" select="$link"/>
                    </xsl:call-template>
                </xsl:when>
                <xsl:when test='$type = "player"'>
                    <img class="enclosure file" src="{$imgPath}/{$language}/bt_video_on.png" type="nsb" url="{$url}" mimetype="{$type}" alt="{$title}" title="{$title}"/>
                </xsl:when>
                <xsl:otherwise>
                    <img class="enclosure file" src="{$imgPath}/{$language}/bt_telecharger_on.png" type="nsb" url="{$url}" mimetype="{$type}" alt="{$text_fichier_joint}" title="{$title}"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:if>
    </xsl:template>

    <xsl:template name="itemShort">
        <xsl:param name="class"/>
        
        <xsl:variable name="link" select="link"/>

        <div class="item short {$class}">
            <div class="t">
                <xsl:call-template name="itemTitle">
                    <xsl:with-param name="link" select="$link"/>
                    <xsl:with-param name="title"><xsl:value-of select='title' disable-output-escaping='yes' /></xsl:with-param>
                </xsl:call-template>
            </div>
            <div class='s'>
                <xsl:for-each select="enclosure">
                    <xsl:call-template name="itemEnclosure">
                        <xsl:with-param name="type">
                            <xsl:call-template name="detectEnclosureType">
                                <xsl:with-param name="url"><xsl:value-of select='@url'/></xsl:with-param>
                                <xsl:with-param name="type"><xsl:value-of select='@type'/></xsl:with-param>
                            </xsl:call-template>
                        </xsl:with-param>
                        <xsl:with-param name="url"><xsl:value-of select='@url'/></xsl:with-param>
                        <xsl:with-param name="title" select='ancestor-or-self::*[title][1]/title'/>
                        <xsl:with-param name="link" select="$link"/>
                    </xsl:call-template>
                </xsl:for-each>
                <!-- yahoo enclosure -->
                <xsl:for-each select="media:content | media:group/media:content[(@type = 'application/x-shockwave-flash') or contains(@type, 'image/') or contains(@type, 'img/') or contains(@type, 'audio/')] | mediadm:group/mediadm:content[(@type = 'application/x-shockwave-flash') or contains(@type, 'image/')  or contains(@type, 'img/') or contains(@type, 'audio/')]">
                    <xsl:call-template name="itemEnclosure">
                        <xsl:with-param name="type">
                            <xsl:call-template name="detectEnclosureType">
                                <xsl:with-param name="type"><xsl:value-of select='@type'/></xsl:with-param>
                                <xsl:with-param name="flashplayer"><xsl:value-of select='@url[../@type = "application/x-shockwave-flash"]'/></xsl:with-param>
                            </xsl:call-template>
                        </xsl:with-param>
                        <xsl:with-param name="url"><xsl:value-of select='@url'/></xsl:with-param>
                        <xsl:with-param name="title"><xsl:value-of select='ancestor-or-self::*[media:title][1]/media:title | ancestor-or-self::*[mediadm:title][1]/mediadm:title'/></xsl:with-param>
                        <xsl:with-param name="link" select="$link"/>
                    </xsl:call-template>
                </xsl:for-each>
                <xsl:for-each select="media:player | media:group/media:player | mediadm:player | mediadm:group/mediadm:player">
                    <xsl:if test='not( ancestor-or-self::*[media:content]) and not( ancestor-or-self::*[mediadm:content]) and not( ancestor-or-self::*[media:group/media:content]) and not( ancestor-or-self::*[mediadm:group/mediadm:content]) and not(ancestor-or-self::*[enclosure]/enclosure/@url)'>
                        <xsl:call-template name="itemEnclosure">
                            <xsl:with-param name="type">player</xsl:with-param>
                            <xsl:with-param name="url"><xsl:value-of select='@url'/></xsl:with-param>
                            <xsl:with-param name="title"><xsl:value-of select='ancestor-or-self::*[media:title][1]/media:title | ancestor-or-self::*[mediadm:title][1]/mediadm:title'/></xsl:with-param>
                            <xsl:with-param name="link" select="$link"/>
                        </xsl:call-template>
                    </xsl:if>
                </xsl:for-each>
            </div>
        </div>
    </xsl:template>

    <xsl:template name="detail_itemIcon">
        <xsl:param name="type"/>
        <xsl:param name="url"/>
        <xsl:param name="title"/>
        <xsl:param name="image"/>
        <xsl:call-template name="itemIcon">
            <xsl:with-param name="type"><xsl:value-of select='$type'/></xsl:with-param>
            <xsl:with-param name="url"><xsl:value-of select='$url'/></xsl:with-param>
            <xsl:with-param name="title"><xsl:value-of select='$title' /></xsl:with-param>
            <xsl:with-param name="image"><xsl:value-of select='$image' /></xsl:with-param>
            <xsl:with-param name="class">
            <xsl:choose>
                <xsl:when test='$image != ""'>thumbnail</xsl:when>
                <xsl:otherwise>icon</xsl:otherwise>
            </xsl:choose>
            </xsl:with-param>
        </xsl:call-template>
    </xsl:template>
  
    <xsl:template name="itemDetail">
        <xsl:param name="class"/>

        <div class="item detail {$class}">
            <!-- Create the icon of the item according to the item enclosure -->
            <xsl:if test='mediadm:group/mediadm:thumbnail/@url | mediadm:thumbnail/@url | media:group/media:thumbnail/@url | media:thumbnail/@url | enclosure/@url[contains( ../@type, "image/") or contains( ../@type, "img/")] != ""'>
                <div class="i" thumbnailLink="{$mod_settings_thumbnailLink}">
                    <xsl:attribute name='thumbnail'><xsl:value-of select='mediadm:group/mediadm:thumbnail/@url | mediadm:thumbnail/@url | media:group/media:thumbnail/@url | media:thumbnail/@url | enclosure/@url[contains( ../@type, "image/") or contains( ../@type, "img/")]'/></xsl:attribute>
                </div>
            </xsl:if>
            <div class="t">
                <xsl:call-template name="itemTitle">
                    <xsl:with-param name="link" select="nsb:normalizeJScriptString(link)"/>
                    <xsl:with-param name="title"><xsl:value-of select='title' disable-output-escaping='yes'/></xsl:with-param>
                </xsl:call-template>
            </div>
            <div class="d">
                <xsl:call-template name="itemNews">
                    <xsl:with-param name="position" select="position()"></xsl:with-param>
                    <xsl:with-param name="data"><xsl:call-template name="itemDescription" /></xsl:with-param>
                </xsl:call-template>
            </div>
            <div class="s">
                <xsl:choose>
                    <xsl:when test="count(enclosure) = 0
                                    and count(media:content | media:group/media:content[(@type = 'application/x-shockwave-flash') or contains(@type, 'image/') or contains(@type, 'img/') or contains(@type, 'audio/')] | mediadm:group/mediadm:content[(@type = 'application/x-shockwave-flash') or contains(@type, 'image/')  or contains(@type, 'img/') or contains(@type, 'audio/')]) = 0
                                    and count(media:player | media:group/media:player | mediadm:player | mediadm:group/mediadm:player) = 0">
                        <xsl:call-template name="itemReadAll">
                            <xsl:with-param name="link" select="link"/>
                        </xsl:call-template>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:variable name="link" select="link"/>
                        
                        <xsl:for-each select="enclosure">
                            <xsl:call-template name="itemEnclosure">
                                <xsl:with-param name="type">
                                <xsl:call-template name="detectEnclosureType">
                                    <xsl:with-param name="type"><xsl:value-of select='@type'/></xsl:with-param>
                                <xsl:with-param name="flashplayer"><xsl:value-of select='@url[../@type = "application/x-shockwave-flash"]'/></xsl:with-param>
                            </xsl:call-template>
                            </xsl:with-param>
                                <xsl:with-param name="url"><xsl:value-of select='@url'/></xsl:with-param>
                                <xsl:with-param name="title" select='ancestor-or-self::*[title][1]/title'/>
                                <xsl:with-param name="link" select="nsb:normalizeJScriptString(ancestor-or-self::*[link][1]/link)"/>
                            </xsl:call-template>
                        </xsl:for-each>
                        <!-- yahoo enclosure -->
                        <xsl:for-each select="media:content | media:group/media:content[(@type = 'application/x-shockwave-flash') or contains(@type, 'image/') or contains(@type, 'img/') or contains(@type, 'audio/')] | mediadm:group/mediadm:content[(@type = 'application/x-shockwave-flash') or contains(@type, 'image/')  or contains(@type, 'img/') or contains(@type, 'audio/')]">
                            <xsl:call-template name="itemEnclosure">
                                <xsl:with-param name="type">
                                    <xsl:call-template name="detectEnclosureType">
                                        <xsl:with-param name="type"><xsl:value-of select='@type'/></xsl:with-param>
                                        <xsl:with-param name="flashplayer"><xsl:value-of select='@url[../@type = "application/x-shockwave-flash"]'/></xsl:with-param>
                                    </xsl:call-template>
                                </xsl:with-param>
                                <xsl:with-param name="url"><xsl:value-of select='@url'/></xsl:with-param>
                                <xsl:with-param name="title"><xsl:value-of select='ancestor-or-self::*[media:title][1]/media:title | ancestor-or-self::*[mediadm:title][1]/mediadm:title'/></xsl:with-param>
                                <xsl:with-param name="link" select="nsb:normalizeJScriptString(ancestor-or-self::*[link][1]/link)"/>
                            </xsl:call-template>
                        </xsl:for-each>
                        <xsl:for-each select="media:player | media:group/media:player | mediadm:player | mediadm:group/mediadm:player">
                            <xsl:if test='not( ancestor-or-self::*[media:content]) and not( ancestor-or-self::*[mediadm:content]) and not( ancestor-or-self::*[media:group/media:content]) and not( ancestor-or-self::*[mediadm:group/mediadm:content]) and not(ancestor-or-self::*[enclosure]/enclosure/@url)'>
                                <xsl:call-template name="itemEnclosure">
                                    <xsl:with-param name="type">player</xsl:with-param>
                                    <xsl:with-param name="url"><xsl:value-of select='@url'/></xsl:with-param>
                                    <xsl:with-param name="title"><xsl:value-of select='ancestor-or-self::*[media:title][1]/media:title | ancestor-or-self::*[mediadm:title][1]/mediadm:title'/></xsl:with-param>
                                    <xsl:with-param name="link" select="nsb:normalizeJScriptString(ancestor-or-self::*[link][1]/link)"/>
                                </xsl:call-template>
                            </xsl:if>
                        </xsl:for-each>
                    </xsl:otherwise>
                </xsl:choose>
            </div>
        </div>
    </xsl:template>
    
    <xsl:template name="itemDescription">
        <xsl:value-of select="description" disable-output-escaping='yes'/><br/><![CDATA[&nbsp;]]>
    </xsl:template>

    <xsl:template name="itemNews">
        <xsl:param name="data"/>
        <xsl:param name="position"/>

        <xsl:choose>
            <xsl:when test="$position = 1">
                <xsl:call-template name="iso2utf8">
                    <xsl:with-param name="data">
                        <xsl:call-template name="truncate_text">
                            <xsl:with-param name="text">
                                <xsl:call-template name="strip_HTML">
                                    <xsl:with-param name="value"><xsl:value-of select="$data" disable-output-escaping='yes'/></xsl:with-param>
                                </xsl:call-template>
                            </xsl:with-param>
                            <xsl:with-param name="length"><xsl:value-of select="$max_item_news_length_first"/></xsl:with-param>
                        </xsl:call-template>
                    </xsl:with-param>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:call-template name="iso2utf8">
                    <xsl:with-param name="data">
                        <xsl:call-template name="truncate_text">
                            <xsl:with-param name="text">
                                <xsl:call-template name="strip_HTML">
                                    <xsl:with-param name="value"><xsl:value-of select="$data" disable-output-escaping='yes'/></xsl:with-param>
                                </xsl:call-template>
                            </xsl:with-param>
                            <xsl:with-param name="length"><xsl:value-of select="$max_item_news_length"/></xsl:with-param>
                        </xsl:call-template>
                    </xsl:with-param>
                </xsl:call-template>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template name="normalizeString">
        <xsl:param name="data"/>
        
        <xsl:call-template name="iso2utf8">
            <xsl:with-param name="data">
                <xsl:call-template name="strip_HTML">
                    <xsl:with-param name="value"><xsl:value-of select="$data" disable-output-escaping='yes'/></xsl:with-param>
                </xsl:call-template>
            </xsl:with-param>
        </xsl:call-template>
    </xsl:template>

    <xsl:template name="iso2utf8">
        <xsl:param name="data"/>
        
        <xsl:value-of select="translate($data,
                            '&#xE8;&#xE9;&#xEA;&#x80;&#x82;&#x83;&#x84;&#x85;&#x86;&#x87;&#x88;&#x89;&#x8A;&#x8B;&#x8C;&#x8E;&#x91;&#x92;&#x93;&#x94;&#x95;&#x96;&#x97;&#x98;&#x99;&#x9A;&#x9B;&#x9C;&#x9E;&#x9F;',
                            'èéê&#x20AC;&#x201A;&#x192;&#x201E;&#x2026;&#x2020;&#x2021;&#x2C6;&#x2030;&#x160;&#x2039;&#x152;&#x17D;&#x2018;&#x2019;&#x201C;&#x201D;&#x2022;&#x2013;&#x2014;&#x2DC;&#x2122;&#x161;&#x203A;&#x153;&#x17E;&#x178;&#xC38A;')"
                            disable-output-escaping='yes' />
    </xsl:template>

    <xsl:template name='truncate_text'>
        <xsl:param name="text"/>
        <xsl:param name="length"/>
        
        <xsl:choose>
            <xsl:when test="string-length($text) &lt; $length">
                <xsl:value-of select="$text" disable-output-escaping='yes'/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="substring($text,1,$length - 3)" disable-output-escaping='yes'/><xsl:text> ...</xsl:text>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <!--
    strip_HTML
    removes all HTML from a given value (including tags that may be left open-ended when returned by the db)
    -->
    <xsl:template name="strip_HTML">
        <xsl:param name="value"/>
        
        <xsl:choose>
            <xsl:when test="contains($value,'&lt;')">
                <xsl:value-of select="substring-before($value,'&lt;')" disable-output-escaping="yes"/>
                <xsl:choose>
                    <xsl:when test="contains(substring-after($value,'&lt;'),'&gt;')">
                        <xsl:call-template name="strip_HTML">
                            <xsl:with-param name="value"><xsl:value-of select="substring-after($value,'&gt;')" disable-output-escaping='yes'/></xsl:with-param>
                        </xsl:call-template>
                    </xsl:when>
                </xsl:choose>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$value" disable-output-escaping="yes"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
  
    <xsl:template name="itemReadAll">
        <xsl:param name="link"/>
        
        <xsl:choose>
            <xsl:when test="$mod_settings_openUrl = '1' and $isWindowsSticker = '0'">
                <a href="#1" onclick="App.getModule('{$mod_id}').contentObj.displayInterface('{$link}')"><xsl:value-of select='$text_read_item'/></a>
            </xsl:when>
            <xsl:otherwise>
                <xsl:choose>
                    <xsl:when test="($link != '') and ($link != '#')">
                        <xsl:choose>
                            <xsl:when test="contains($link, 'javascript:nsx')">
                                <a href="#null" link='{$link}' onclick="window.external.navigate('{$link}')"><xsl:value-of select='$text_read_item'/></a>
                            </xsl:when>
                            <xsl:otherwise>
                                <a href="#null" link='{$link}' onclick="window.external.navigate('{$link}')"><xsl:value-of select='$text_read_item'/></a>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:when>
                </xsl:choose>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
</xsl:stylesheet>
