function FlashVars(data)
{
    var m_vars = {};

    this.toString = function()
    {
        var str = "";
    
        for(var name in m_vars)
        {
            if (str)
            {
                str += "&"
            }
            
            str += name + "=" + encodeURIComponent(m_vars[name]);
        }
        
        return str;
    }
    
    this.variable = function(name,value)
    {
        if (typeof(value) == "undefined")
        {
            return m_vars[name];
        }
        else
        {
            m_vars[name] = value;
            
            return this;
        }
    }
    
    this.variables = function()
    {
        return m_vars;
    }
    
    this.load = function(data)
    {
        if (!data)
        {
            return;
        }
    
        switch(typeof(data))
        {
            case "string":
            {
                var vars = data.replace("&amp;","&").split("&");
                
                for(var i = 0; i < vars.length; ++ i)
                {
                    var entry = vars[i].split("=");
                    
                    m_vars[entry[0]] = decodeURIComponent(entry[1]);
                }
            }
            break;
            case "object":
            {
                for(var name in data)
                {
                    m_vars[name] = data[name];
                }
            }
            break;
            default:
            {
                throw new Error("Unexcepted data.");
            }
        }
    }
    
    this.load(data);
}

function FlashObject(sticker,mid,flags)
{
    var widgetNode = mid ? Tabs.getTabByModuleIdEx(mid) : null;
    
    var type = widgetNode ? widgetNode.node.getAttribute("type") : null; 

    var url = widgetNode ? widgetNode.node.getAttribute("url") : null;
    
    if (url)
    {
        type = url;
    } 
    
    var m_flashVars = new FlashVars({platform:"nosibox"});
    
    var self = this;

    function error(msg)
    {
        logger("FLASH").error(msg);
    }
    
    function trace(msg)
    {
        logger("FLASH").trace(msg);
    }

    function warning(msg)
    {
        logger("FLASH").warning(msg);
    }

    function toXML(v) {
        switch(typeof(v)) {
            case "string": return "<string>"+v+"</string>";
            case "number": return "<number>"+v+"</number>";        
            case "boolean": return v ? "<true/>" : "<false/>";
            case "undefined": return "<undefined/>";
            case "object":{
                if (!v) {
                    return "<null/>";
                } else if (libJS.isArray(v)) {
                    var xml = '';
                    for(var i = 0; i < v.length; ++ i) xml += '<property id="' + i + '">' + toXML(v[i]) + '</property>';
                    return '<array>' + xml + '</array>';
                } else {
                    var xml = '';
                    for(var key in v) xml += '<property id="' + key + '">' + toXML(v[key]) + '</property>';
                    return '<object>' + xml + '</object>';
                }            
            }
        }
    }
    
    function toJScript(node) {
        switch(node.nodeName)
        {
            case "null": return null;
            case "true": return true;
            case "false": return false;
            case "string": return node.text;
            case "number": return libJS.toNum( node.text);
            case "array": {
                var result = [];
                var childNodes = node.childNodes;
                for(var i = 0; i < childNodes.length; ++ i) {
                    var childNode = childNodes[i];
                    result.push( toJScript( childNode.childNodes[0]));                            
                }
                
                return result;
            }
            break;
            case "object": {
                var result = {};
                var childNodes = node.childNodes;
                for(var i = 0; i < childNodes.length; ++ i) {
                    var childNode = childNodes[i];
                    result[childNode.getAttribute("id")] = toJScript( childNode.childNodes[0]);                            
                }
                
                return result;
            }
            break;
            case "undefined": return;
            default: {
                warning("Unexcepted flash type. ('" + node.nodeName + "')")
                return null;
            }
        }
    }

    //
    // New API to manage sentinels
    //
    // See "lib_js_comp" module to get more information about sentinel or wiki:Sentinel
    function MySentinel(mySentinel) {
        this.feed = function(def) {
            if (typeof(def) == "undefined") {
                return mySentinel.feed().definition(); // return a json object of the feed 
            } else {
                mySentinel.feed(def);
                return this;
            }
        }
        this.match = function(type,def) {
            if (typeof(type) == "undefined") {
                return mySentinel.match().definition(); // return a json object of the match
            } else {
                mySentinel.match(type, def);
                return this;
            }
        }
        this.filter = function(type, def) {
            if (typeof(type) == "undefined") {
                return mySentinel.filter().definition(); // return a json object of the filter
            } else {
                mySentinel.filter(type, def);
                return this;
            }
        }
        this.enable = function(b) {
            if (typeof(b) == "undefined") {
                return mySentinel.enable();
            } else {
                mySentinel.enable(b);
                return this;
            }
        }
        this.getValue = function(key) {
            return mySentinel.param(key);
        } 
        this.setValue = function(key, value) {
            mySentinel.param(key, value);
            return this;
        }
        //
        // Actions
        //
        /*
          \@brief create alert from an xslt document who transform the sentinel result into the html content of the alert
          \@param def : object  
          {
            xslt: xslt document
            width : width of the alert
            height : height of the alert
            displayTime : display time of the alert in sec
          }
        */
        this.doAlert = function(def) {
            mySentinel.doAlert(def);
            return this;
        }
        this.undoAlert = function() {
            mySentinel.undoAlert();
            return this;
        }
        /*
          \@brief create alert from a nosibay alert feed (c.f. wiki https://carne.nosibay.com/wiki/index.php/Alertes_-_Infobulles#Flux_d.27alertes)
          \@param def : object  
          {
            xslt: xslt document
            paramsXslt : parameters of the xslt document 
          }
        */
        this.doAlertEx = function(def) {
            mySentinel.doAlertEx(def);
            return this;
        }
        this.undoAlertEx = function() {
            mySentinel.undoAlertEx();
            return this;
        }
        this.doAction = function(type, def) {
            mySentinel.doAction(type, def);
            return this;
        }        
        this.undoAction = function(type, def) {
            mySentinel.undoAction(type, def);
            return this;
        }        
        // remove the sentinel
        this.dispose = function() {
            mySentinel.dispose();
        }
    }
    
    var stickerapi={
        callDeferedCommand:function() {
            warning("callDeferedCommand not implemented."); 
        },
        setTitle:function(title) {
            sticker.text=title;
        },
        setCaption:function() {
            if (flags.ignoreCaption) {
                return;
            }
            sticker.setCaption(false);
        },
        minimize:function() {
            sticker.minimize();
        },
        close:function() {                
            sticker.close("OK");
        },
        pin:function(b) {
            sticker.topmost=b;
        },
        setTabValue:function(n,v) {
            if (!widgetNode) {
                return;
            }
            
            libJS.Xml.setValue(widgetNode.node,"settings/" + n, v);

            Tabs.saveTabByTabId(widgetNode.name);            
        },
        getTabValue:function(n) {
            try {
                if (!widgetNode) {
                    return;
                }
                // return attribute if it exists
                if(/^[a-zA-Z0-9]+$/.test(n) && widgetNode.node.getAttribute(n))
                    return widgetNode.node.getAttribute(n);
            
                return libJS.Xml.getValue(widgetNode.node,"settings/" + n, null);
            }
            catch(err) {
                warning(err.message);
            }
        },
        setValue:function(n, v) { 
            return this.setTabValue(n,v);
        },
        getValue:function(n) {
            // return tab setting, specific registry value or unspecific registry value in order
            return this.getTabValue(n) || libJS.Registry.getValue((mid + "/" + n).replace(/\//g,"\\")) || libJS.Registry.getValue(n.replace(/\//g,"\\"));
        }
    }
    
    function getAbsoluteUrl(url) {
        if (url.indexOf("http") == 0) // http://...
        {
          return url;
        }
        else if (/^modules[/\\]/.test(url)) // modules/....
        {
          return url;
        }
        else if ((url.length >= 2) && (url.charAt(1) == ':')) // c:/...
        {
          return url;
        }
        else
        {
          return "modules\\stickers\\" + type +"\\" + url;
        }
    }
    
    function createFeedConfig(feed) {
        feed.type = feed.type || "binary";

        if (feed.url) {
            feed.url = getAbsoluteUrl(feed.url);
        }
        
        if (!feed.id) {
            feed.id = feed.url;
        }            
        
        return feed;
    }

    var api={
        my:{
            application:{
                activateSticker:function(mid) {
                    stickers.activateStickerById(mid);
                },
                showSettings:function() {
                    settingsWindow.open();
                },
                showAlertsHistory:function() {
                    historyWindow.open();
                }
            },
            browser:{
                openURL:function(url) {
                    (system.env.browser || system.getObject("BROWSER")).navigate(url);
                },
                showPopup:function(url,width,height)
                {
	                var window = system.createWindow( libJS.Conf.toString("IDC_PATH_LAYOUT_POPUP", libJS.Conf.toString("IDC_PATH_LAYOUT_BALLOON")), {});

                	if ((url.charAt(0) == "<") && (url.charAt(url.length - 1) == ">"))
                	{
                		var idPopup = 0; var popupPath;
                		
                		libJS.fso.CreateFolder("cache/popup");
                		
                		do
                		{
                			popupPath = "cache/popup/" + (idPopup ++) + ".html"
						}
						while(libJS.fso.fileExists(popupPath));
                	
                		libJS.fso.writeTextFile(popupPath,url);

		                window.addEventListener( "Close", function(exitCode)
		                {
	                		libJS.fso.deleteFile(popupPath);
		                });
                		
                		url = "local://" + popupPath;
					}
                
	                window.width   = width;
	                window.height  = height;
	                window.topmost = true;
	                window.center();
	                
					window.items.defaultView.Navigate( url);    
					
					window.visible = true;                     	
				}
            },
            // id : reader id
            // feed : jscript object of the feed description
            reader:function(id,feed) {
                var readerId = "flash:" + mid + ":" + id; 
                
                var reader = system.env.readerEngine.getReader(readerId,createFeedConfig(feed));
                
                reader.addListener(new function() {
                    function close() {
                        trace("reader => close : '" + readerId + "'");
                        
                        reader.remListener(readerId); 
                        
                        system.env.readerEngine.deleteReader(readerId, reader);
                    }
                
                    this.onSuccess = function(e) {
                        trace("NosibayReader('" + readerId + "')::onReaderSuccess");
                        
                        if (!self.invoke("onReaderSuccess", [id, {cache:e.cache || e.url}], false)) {
                            close();
                        } 
                    },
                    this.onFailure = function(e) {
                        warning("NosibayReader('" + readerId + "')::onReaderError");

                        if (!self.invoke("onReaderError", [id, {cache:null}], false)) {
                            close();
                        } 
                    },
                    this.onSimilar = function(e) {
                        trace("NosibayReader('" + readerId + "')::onSimilar");

                        if (!self.invoke("onReaderSimilar", [id, {cache:e.cache || e.url}], false)) {
                            close();
                        } 
                    },
                    this.onException = function(e) {
                        warning("NosibayReader('" + readerId + "')::onException");

                        if (!self.invoke("onReaderError", [id, {cache:null}], false)) {
                            close();
                        } 
                    }
                }, 
                readerId);
                
                return true;
            },
            // id : reader id
            // feed : jscript object of the feed description
            // methodName : name of the method called 
            readerCall:function(id,feed,methodName) {
                var readerId = "flash:" + mid + ":" + id;
                
                var reader = system.env.readerEngine.getReader(readerId,createFeedConfig(feed));
                
                if (!reader) {
                    warning("NosibayReader('" + readerId + "')::readerCall : '" + readerId + "' not found.");
                    
                    return;
                }
                
                var methodArgs = [];
                
                for(var i = 3; i < arguments.length; ++ i) methodArgs.push( arguments[i]);
                
                switch(methodName) {
                    case "dispose":
                    {
                        trace("NosibayReader('" + readerId + '")::dispose');
                    
                        reader.remListener(readerId);
                        
                        system.env.readerEngine.deleteReader(readerId,reader);
                    }
                    break;
                    case "close":
                    {
                        trace("NosibayReader('" + readerId + '")::close');
                    
                        reader.remListener(readerId);
                    }
                    break;
                    case "engineRequest":
                    {
                        trace("NosibayReader('" + readerId + '")::engineRequest');
                        
                        return reader.engineRequest.apply(reader, methodArgs);
                    }
                    break;
                    default:
                    {
                        var method = reader[methodName];
                        
                        if (typeof(method) != "function") {
                            warning("'" + methodName + "' is not a method of Reader.");
                            return;
                        }
                    
                        return method.apply( reader, methodArgs);
                    }
                }

                return true;
            },
            // shared = true : the sentinel is shared between all the module instance
            sentinel:function(id, feed, shared) {
            
                var sentinelId = id;
                
                if (feed && feed.url)
                {
                    feed.url = getAbsoluteUrl(feed.url);
                }
                
                // build the sentinel owner id
                var ownerId = (shared != true) ? mid.toLowerCase() : type.toLowerCase();
                
                var sentinel = readerEngine.mySentinels().get(sentinelId);
                
                if (!sentinel) { // create a new sentinel
                    sentinel = readerEngine.sentinel( sentinelId, feed, {owner:ownerId}, {directory:directory});
                }
                
                if (sentinel == null) {
                  return null;
                }
                
                // check that the sentinel belongs to this sticker or sticker class
                if ((sentinel.attr("owner") != mid.toLowerCase()) && (sentinel.attr("owner") != type.toLowerCase())) {
                    warning("not owner. (owner : " + sentinel.attr("owner") + ", mid : " + mid + ", type : " + type);
                    return null;
                } 
               
                return new MySentinel( sentinel);
            },
            sentinelCall:function(id,methodName) {
                var sentinel = api.my.sentinel(id);
                
                if (!sentinel) {
                    warning("Sentinel '" + id + "' not found.");
                    
                    return;
                }
                
                var method = sentinel[methodName];
                
                if (typeof(method) != "function") {
                    warning("'" + methodName + "' is not a method of MySentinel.");
                    return;
                }
                
                var methodArgs = [];
                
                for(var i = 2; i < arguments.length; ++ i) methodArgs.push( arguments[i]);
                
                return method.apply( sentinel, methodArgs);
            },
            sentinels:
            {
                getSentinelCount:function() {
                    return readerEngine.mySentinels().filter( function(id, sentinel) {
                        // check that the sentinel belongs to this sticker or sticker class
                        return (sentinel.attr("owner") == mid.toLowerCase()) || (sentinel.attr("owner") == type.toLowerCase());
                    }).size();
                },
                getSentinelAt:function(index) {
                    var sentinel = readerEngine.mySentinels().find(function(id,sentinel) {
                        if ((sentinel.attr("owner") != mid.toLowerCase()) && (sentinel.attr("owner") != type.toLowerCase())) {
                            return false;
                        }
                    
                        return index -- == 0; 
                    });
                    
                    if (!sentinel) {
                      return null;
                    }
                    
                    return sentinel.getId();
                },
                isSentinelExist:function(id) {
                    return readerEngine.mySentinels().get(id) != null;
                }
            },           
            alert:{
                create:function(description) {
                    libJS.alertEngine.create(description);
                },
                dispose:function(id) {
                    libJS.alertEngine.alert(id).dispose();
                }
            },         
            network:{
                // get a file periodically and call the flash after each request
                subscribe:function(idRequest, uri, period) {
                    trace("stickers.api.network.subscribe("+idRequest+"," + uri + "," + period + ")");
                    var idApi = mid + "-" + idRequest;
                    var url = uri;
                    try {
                      var oUrl = new libJS.Url(uri);
                      url = (oUrl.getProtocol() == "reg") ? libJS.Registry.toString( (oUrl.getHost() + oUrl.getPath()).replace(new RegExp("/","g"), "\\"), oUrl.getParameters()["default"]) : uri;
                    }                            
                    catch(err) {
                      url = uri;
                    }
                    var feed   = {url:url, type:"cache_xml"};
                    var reader = system.env.readerEngine.getReader(idApi, feed);
                    var listener = new function()
                    {
                        this.onSuccess   = function(e) { trace("stickers.api.network.onSuccess("+idRequest+","+e.cache+")"); self.invoke("onRequestComplete", [idRequest, url, e.cache], false); },
                        this.onFailure   = function(e) { trace("stickers.api.network.onFailure("+idRequest+","+url+")");     self.invoke("onRequestFailed", [idRequest, url], false); },
                        this.onSimilar   = function(e) { trace("stickers.api.network.onSimilar("+idRequest+","+e.cache+")"); self.invoke("onRequestComplete", [idRequest, url, e.cache], false); },
                        this.onException = function(e) { trace("stickers.api.network.onException("+idRequest+","+url+")");   self.invoke("onRequestFailed", [idRequest, url], false); }
                    }
                    reader.addListener(listener,idApi);

                    period = parseInt(period,10);
                    if (!isNaN(period)) {
                      system.env.jsScheduler.schedule("interval", {value:Math.round(period/1000),dieOnExit:true}, idApi).doAction("reader-request", {reader:feed}).enable(true);
                      reader.engineRequest(function() {},{force:false},idApi);
                    }
                    else {
                      reader.engineRequest(function() {},{force:false},idApi);
                    }
                },
                // stop monitoring a file
                unsubscribe:function(idRequest) {
                    trace("stickers.api.network.unsubscribe(" + idRequest + ")");
                    var idApi = mid + "-" + idRequest;
                    system.env.readerEngine.getReaderByRef( idApi).forEach(function(id) {
                      var reader = this.getObject();
                      reader.setTtl( 0, idApi);
                      system.env.jsScheduler.progs().filter(function() { return this.getId() == idApi; }).forEach(function() { this.dispose(); });
                      reader.abortRequest( idApi);
                      reader.remListenerById( idApi);
                      system.env.readerEngine.deleteReader( idApi, reader);
                    });
                }
            },
            trace:function(txt) {
                trace(txt);
            },
            error:function(txt) {
                error(txt);
            },
            locale:function(ids) {
                return system.createObject("INFO").Value(ids);
            },
            sticker:stickerapi
        },
        sticker:stickerapi
    }
    
    sticker.items.flash.addEventListener('FlashCall', function(desc) 
    {
        var doc = libJS.Xml.loadDocument(desc);
        var methodName=doc.selectSingleNode('invoke/@name').text;
        var args=[];
        var argsXml=doc.selectNodes('invoke/arguments/*');
        for(var i=0;i<argsXml.length;i++) {
            args.push(toJScript(argsXml[i]));
        }            

        // find the method called in the api object
        // I could eval but this is safer as that can only match methods in the api object
        var obj = null; var method=api; 
        subStr=methodName;
        while(subStr && method) {
            var m=subStr.match(/(\w*?)(\.|$)(.*)/);
            subStr=m[3]
            obj = method;
            method = obj[m[1]];
        }
        
        if (!method)
        {
            warning("The following jscript function called from the flash not found : '" + methodName + "'"); 
            return;
        }

        sticker.items.flash.SetReturnValue(toXML(method.apply(obj,args)));
    });
    
    // listen sentinel manager event to send insert/remove event to the sticker    
    var sentinelListener = new function() {
        this.onElementInserted = function(e) {
            var sentinel = e.element;
        
            if ((sentinel.attr("owner") != mid) && (sentinel.attr("owner") != type)) {
                return;
            }
            
            self.invoke("onSentinelInserted", [sentinel.getId()]);
        }
        
        this.onElementRemoved = function(e) { 
            var sentinel = e.element;
        
            if ((sentinel.attr("owner") != mid) && (sentinel.attr("owner") != type)) {
                return;
            }
            
            self.invoke("onSentinelRemoved", [sentinel.getId()]);
        }
    }             
    
    readerEngine.mySentinels().addListener( sentinelListener);
    
    sticker.addEventListener("Close",function() {
        readerEngine.mySentinels().remListener(sentinelListener);
         
        try {
            self.invoke("onClose", []);
        } catch(e) {
            error("onClose execution failed ("+e+") on sticker "+mid);
        }
        sticker=null;
    });
    
    this.invoke = function(methodName,args)
    {
        // convert arguments to an XML String
        function argumentsToXML(args) {
            var argsStr="";
    
            for(var i=0;i<args.length;i++) argsStr += toXML(args[i]);            
    
            return argsStr;
        }

        return sticker.items.flash.IShockwaveFlash.CallFunction("<invoke name=\""+methodName+"\" returntype=\"xml\"><arguments>"+argumentsToXML(args)+"</arguments></invoke>")
    }

    this.setProperty = function(name,value)
    {
        sticker.items.flash.IShockwaveFlash[name] = value;
    }
    
    this.setFlashVars = function(value)
    {
        m_flashVars.load(value);
    }
    
    this.loadMovie = function(moviePath)
    {
        m_flashVars.variable('mid',mid);
    
        if (widgetNode)
        {
            var productAppDataDir   = system.createObject("INFO").Value("ProductAppDataDir");
            var applicationPath     = productAppDataDir+"/modules/stickers/"+type+"/";
            var configPath          = applicationPath+"/config.xml";        
            
            m_flashVars.variable('applicationPath',applicationPath);
            m_flashVars.variable('configPath',configPath);
        
            var config=system.createXmlDocument();
            config.load(configPath);
        
            // read flash vars in config and module node
            var flashVarsConfig=config.selectNodes("config/flashVars/*");
            for(var i=0;i<flashVarsConfig.length;i++) {
                m_flashVars.variable(flashVarsConfig.item(i).nodeName,flashVarsConfig.item(i).text);
            }
        
            m_flashVars.load(widgetNode.node.getAttribute("flashVars"));
        
            // check if there is a substitute start for the widget
            var launcherPath=XGetString(config, "config/launch");
            if(launcherPath) 
            {
                m_flashVars.variable('widgetPath',moviePath);
                
                if(/^\.\//.test(launcherPath))
                    moviePath=applicationPath+launcherPath;
                else
                    moviePath=productAppDataDir+"/"+launcherPath;
            } 
            else 
            {
                moviePath=applicationPath+"widget.swf";
            }
        }

        sticker.items.flash.IShockwaveFlash.flashVars = m_flashVars.toString();
        
        sticker.items.flash.IShockwaveFlash.Movie = libJS.fso.GetAbsolutePathName(moviePath);
    }
}                           
